﻿using Microsoft.Extensions.Logging;
using Serilog.Filters;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Reflection.PortableExecutable;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.DataFormats;

namespace AntiAircraftGun
{
    public partial class FormMapWithSetAntiAircraftGuns : Form
    {
        /// <summary>
        /// Словарь для выпадающего списка
        /// </summary>
        private readonly Dictionary<string, AbstractMap> _mapsDict = new()
        {
            { "Простая карта", new SimpleMap()},  { "Простая карта2", new SimpleMap2()}
        };
        /// <summary>
        /// Объект от коллекции карт
        /// </summary>
        private readonly MapsCollection _mapsCollection;
        /// <summary>
        /// Логер
        /// </summary>
        private readonly ILogger _logger;
        /// Конструктор
        /// </summary>
        public FormMapWithSetAntiAircraftGuns(ILogger<FormMapWithSetAntiAircraftGuns> logger)
        {
            InitializeComponent();
            _logger = logger;
            _mapsCollection = new MapsCollection(pictureBox.Width, pictureBox.Height);
            comboBoxSelectorMap.Items.Clear();
            foreach (var elem in _mapsDict)
            {
                comboBoxSelectorMap.Items.Add(elem.Key);
            }
        }
        /// <summary>
        /// Заполнение listBoxMaps
        /// </summary>
        private void ReloadMaps()
        {
            int index = listBoxMaps.SelectedIndex;
            listBoxMaps.Items.Clear();
            for (int i = 0; i < _mapsCollection.Keys.Count; i++)
            {
                listBoxMaps.Items.Add(_mapsCollection.Keys[i]);
            }
            if (listBoxMaps.Items.Count > 0 && (index == -1 || index >= listBoxMaps.Items.Count))
            {
                listBoxMaps.SelectedIndex = 0;
            }
            else if (listBoxMaps.Items.Count > 0 && index > -1 && index < listBoxMaps.Items.Count)
            {
                listBoxMaps.SelectedIndex = index;
            }
        }
        /// <summary>
        /// Добавление карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddMap_Click(object sender, EventArgs e)
        {
            if (comboBoxSelectorMap.SelectedIndex == -1 || string.IsNullOrEmpty(textBoxNewMapName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogInformation("При добавлении карты {0}", comboBoxSelectorMap.SelectedIndex == -1 ? "не была выбрана карта" : "не была названа карта");
                return;
            }
            if (!_mapsDict.ContainsKey(comboBoxSelectorMap.Text))
            {
                MessageBox.Show("Нет такой карты", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogInformation($"Попытка добавить несуществующую карту: {textBoxNewMapName.Text}");
                return;
            }
            _mapsCollection.AddMap(textBoxNewMapName.Text, _mapsDict[comboBoxSelectorMap.Text]);
            ReloadMaps();
            _logger.LogInformation($"Добавлена карта {textBoxNewMapName.Text}");
        }
        /// <summary>
        /// Выбор карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListBoxMaps_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
            _logger.LogInformation($"Переход на карту: {listBoxMaps.SelectedItem?.ToString() ?? string.Empty}");
        }
        /// <summary>
        /// Удаление карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDeleteMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                _logger.LogInformation($"Попытка удалить несуществующую карту: {textBoxNewMapName.Text}");
                return;
            }
            if (MessageBox.Show($"Удалить карту {listBoxMaps.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _mapsCollection.DelMap(listBoxMaps.SelectedItem?.ToString() ?? string.Empty);
                ReloadMaps();
                _logger.LogInformation($"Удалена карта: {listBoxMaps.SelectedItem?.ToString() ?? string.Empty}");
            }
        }
        /// <summary>
        /// Добавление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddAntiAircraftGun_Click(object sender, EventArgs e)
        {
            var formAntiAircraftGunConfig = new FormAntiAircraftGunConfig();
            formAntiAircraftGunConfig.AddEvent(AddAntiAircraftGun);
            formAntiAircraftGunConfig.Show();
        }
        private void AddAntiAircraftGun(DrawingAntiAircraftGun drawingAntiAircraftGuns)
        {
            try
            {
                if (listBoxMaps.SelectedIndex == -1)
                {
                    _logger.LogInformation($"Попытка добавления объекта на невыбранную карту");
                    return;
                }
                if (drawingAntiAircraftGuns == null)
                {
                    MessageBox.Show("Нужно выбрать объект перед добавлением");
                    _logger.LogInformation($"Не выбран объект для добавления на карту");
                    return;
                }
                DrawingObjectAntiAircraftGun antiAircraftGun = new DrawingObjectAntiAircraftGun(drawingAntiAircraftGuns);
                if (_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] + antiAircraftGun != -1)
                {
                    pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
                    MessageBox.Show("Объект добавлен");
                    _logger.LogInformation($"Добавлен объект {drawingAntiAircraftGuns} на карту ");
                }
                else
                {
                    MessageBox.Show("Не получилось добавить объект");
                    _logger.LogInformation($"Не получилось добавить объект {drawingAntiAircraftGuns} на карту ");
                }
            }
            catch (StorageOverflowException ex)
            {
                _logger.LogWarning($"Ошибка переполнения хранилища: {ex.Message}");
                MessageBox.Show($"Ошибка переполнения хранилища: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// Удаление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveAntiAircraftGun_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text))
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            try
            {
                if (_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] - pos != null)
                {
                    MessageBox.Show("Объект удален");
                    pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
                    _logger.LogInformation($"Удален объект {pos}");
                }
                else
                {
                    MessageBox.Show("Не удалось удалить объект");
                    _logger.LogInformation($"Не удалось удалить объект {pos}");
                }
            }
            catch (AntiAircraftGunNotFoundException ex)
            {
                _logger.LogWarning($"Ошибка удаления: {ex.Message}");
                MessageBox.Show($"Ошибка удаления {ex.Message}");
            }
            catch (Exception ex)
            {
                _logger.LogWarning($"Неизвестная ошибка: {ex.Message}");
                MessageBox.Show($"Неизвестная ошибка {ex.Message}");
            }
        }
        /// <summary>
        /// Вывод набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowStorage_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
        }
        /// <summary>
        /// Вывод карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowOnMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowOnMap();
        }
        /// <summary>
        /// Перемещение
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            //получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "buttonUp":
                    dir = Direction.Up;
                    break;
                case "buttonDown":
                    dir = Direction.Down;
                    break;
                case "buttonLeft":
                    dir = Direction.Left;
                    break;
                case "buttonRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].MoveObject(dir);
        }
        /// <summary>
        /// Обработка нажатия "Сохранение"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Успешно сохранено в файл: {saveFileDialog.FileName}");
                }
                catch(Exception ex)
                {
                    MessageBox.Show($"Не сохранилось {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Ошибка сохранения в файл: {saveFileDialog.FileName}");
                }
            }
        }
        /// <summary>
        /// Обработка нажатия "Загрузка"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    ReloadMaps();
                    _logger.LogInformation($"Успешная загрузка из файла: {openFileDialog.FileName}");
                }
                catch(Exception ex)
                {
                    MessageBox.Show($"Не загрузилось {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Ошибка загрузки из файла: {openFileDialog.FileName}");
                }
            }
        }
        /// <summary>
        /// Сортировка по типу
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSortByType_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].Sort(new AntiAircraftGunCompareByType());
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
        }
        /// <summary>
        /// Сортировка по цвету
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSortByColor_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].Sort(new AntiAircraftGunCompareByColor());
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
        }
    }
}
