﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AntiAircraftGun
{
    /// <summary>
    /// Карта с набром объектов под нее
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <typeparam name="U"></typeparam>
    internal class MapWithSetAntiAircraftGunsGeneric<T, U>
        where T : class, IDrawingObject, IEquatable<T>
        where U : AbstractMap
    {
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Размер занимаемого объектом места (ширина)
        /// </summary>
        private readonly int _placeSizeWidth = 150;
        /// <summary>
        /// Размер занимаемого объектом места (высота)
        /// </summary>
        private readonly int _placeSizeHeight = 90;
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly SetAntiAircraftGunsGeneric<T> _setAntiAircraftGuns;
        /// <summary>
        /// Карта
        /// </summary>
        private readonly U _map;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="picWidth"></param>
        /// <param name="picHeight"></param>
        /// <param name="map"></param>
        public MapWithSetAntiAircraftGunsGeneric(int picWidth, int picHeight, U map)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _setAntiAircraftGuns = new SetAntiAircraftGunsGeneric<T>(width * height);
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _map = map;
        }
        /// <summary>
        /// Перегрузка оператора сложения
        /// </summary>
        /// <param name="map"></param>
        /// <param name="antiAircraftGuns"></param>
        /// <returns></returns>
        public static int operator +(MapWithSetAntiAircraftGunsGeneric<T, U> map, T antiAircraftGuns)
        {
            return map._setAntiAircraftGuns.Insert(antiAircraftGuns);
        }
        /// <summary>
        /// Перегрузка оператора вычитания
        /// </summary>
        /// <param name="map"></param>
        /// <param name="position"></param>
        /// <returns></returns>
        public static T operator -(MapWithSetAntiAircraftGunsGeneric<T, U> map, int position)
        {
            return map._setAntiAircraftGuns.Remove(position);
        }
        /// <summary>
        /// Вывод всего набора объектов
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowSet()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawAntiAircraftGuns(gr);
            return bmp;
        }
        /// <summary>
        /// Просмотр объекта на карте
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowOnMap()
        {
            Shaking();
            foreach (var antiAircraftGun in _setAntiAircraftGuns.GetAntiAircraftGuns())
            {
                return _map.CreateMap(_pictureWidth, _pictureHeight, antiAircraftGun);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// Перемещение объекта по крате
        /// </summary>
        /// <param name="direction"></param>
        /// <returns></returns>
        public Bitmap MoveObject(Direction direction)
        {
            if (_map != null)
            {
                return _map.MoveObject(direction);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// Получение данных в виде строки
        /// </summary>
        /// <param name="sep"></param>
        /// <returns></returns>
        public string GetData(char separatorType, char separatorData)
        {
            string data = $"{_map.GetType().Name}{separatorType}";
            foreach (var antiAircraftGun in _setAntiAircraftGuns.GetAntiAircraftGuns())
            {
                data += $"{antiAircraftGun.GetInfo()}{separatorData}";
            }
            return data;
        }
        /// <summary>
        /// Загрузка списка из массива строк
        /// </summary>
        /// <param name="records"></param>
        public void LoadData(string[] records)
        {
            foreach (var rec in records)
            {
                _setAntiAircraftGuns.Insert(DrawingObjectAntiAircraftGun.Create(rec) as T);
            }
        }
        /// <summary>
        /// Сортировка
        /// </summary>
        /// <param name="comparer"></param>
        public void Sort(IComparer<T> comparer)
        {
            _setAntiAircraftGuns.SortSet(comparer);
        }
        /// <summary>
        /// "Взбалтываем" набор, чтобы все элементы оказались в начале
        /// </summary>
        private void Shaking()
        {
            int j = _setAntiAircraftGuns.Count - 1;
            for (int i = 0; i < _setAntiAircraftGuns.Count; i++)
            {
                if (_setAntiAircraftGuns[i] == null)
                {
                    for (; j > i; j--)
                    {
                        var antiAircraftGun = _setAntiAircraftGuns[j];
                        if (antiAircraftGun != null)
                        {
                            _setAntiAircraftGuns.Insert(antiAircraftGun, i);
                            _setAntiAircraftGuns.Remove(j);
                            break;
                        }
                    }
                    if (j <= i)
                    {
                        return;
                    }
                }
            }
        }
        /// <summary>
        /// Метод отрисовки фона
        /// </summary>
        /// <param name="g"></param>
        private void DrawBackground(Graphics g)
        {
            Brush brushGreen = new SolidBrush(Color.Green);
            Brush brushLightGray = new SolidBrush(Color.LightGray);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight; ++j)
                {
                    g.FillRectangle(brushGreen, i * _placeSizeWidth, j * _placeSizeHeight, _placeSizeWidth, _placeSizeHeight);
                    g.FillRectangle(brushLightGray, i * _placeSizeWidth + _placeSizeWidth / 10, j * _placeSizeHeight + _placeSizeHeight / 10, _placeSizeWidth * 8 / 10, _placeSizeHeight * 8 / 10);
                }
            }
        }
        /// <summary>
        /// Метод прорисовки объектов
        /// </summary>
        /// <param name="g"></param>
        private void DrawAntiAircraftGuns(Graphics g)
        {
            int xPositionInStorage = _placeSizeWidth / 10;
            int yPositionInStorage = _placeSizeHeight / 10;
            foreach (var antiAircraftGun in _setAntiAircraftGuns.GetAntiAircraftGuns())
            {
                antiAircraftGun.SetObject(xPositionInStorage, yPositionInStorage, _pictureWidth, _pictureHeight);
                antiAircraftGun.DrawingObjectAntiAircraftGun(g);
                xPositionInStorage += _placeSizeWidth;
                if (xPositionInStorage > _pictureWidth - _placeSizeWidth)
                {
                    xPositionInStorage = _placeSizeWidth / 10;
                    yPositionInStorage += _placeSizeHeight;
                }
            }
        }
    }
}
